@@ -8,7 +8,19 @@
 #include "bwt.h"
 #include "kvec.h"
 
+#ifdef USE_MALLOC_WRAPPERS
+#  include "malloc_wrap.h"
+#endif
+
+typedef struct {
+	bwtint_t k, l;
+} qintv_t;
+
+#define qintv_eq(a, b) ((a).k == (b).k && (a).l == (b).l)
+#define qintv_hash(a) ((a).k>>7^(a).l<<17)
+
 #include "khash.h"
+KHASH_INIT(qintv, qintv_t, uint64_t, 1, qintv_hash, qintv_eq)
 KHASH_MAP_INIT_INT64(64, uint64_t)
 
 #define MINUS_INF -0x3fffffff
@@ -17,7 +29,7 @@ KHASH_MAP_INIT_INT64(64, uint64_t)
 struct __mempool_t;
 static void mp_destroy(struct __mempool_t*);
 typedef struct {
-	uint32_t qk, ql;
+	bwtint_t qk, ql;
 	int I, D, G;
 	uint32_t pj:2, qlen:30;
 	int tlen;
@@ -27,14 +39,14 @@ typedef struct {
 
 #include "ksort.h"
 KSORT_INIT_GENERIC(int)
-#define __hitG_lt(a, b) ((a).G > (b).G)
+#define __hitG_lt(a, b) (((a).G + ((int)(a).n_seeds<<2)) > (b).G + ((int)(b).n_seeds<<2))
 KSORT_INIT(hitG, bsw2hit_t, __hitG_lt)
 
 static const bsw2cell_t g_default_cell = { 0, 0, MINUS_INF, MINUS_INF, MINUS_INF, 0, 0, 0, -1, -1, {-1, -1, -1, -1} };
 
 typedef struct {
 	int n, max;
-	uint32_t tk, tl;
+	uint32_t tk, tl; // this is fine
 	bsw2cell_t *array;
 } bsw2entry_t, *bsw2entry_p;
 
@@ -87,7 +99,7 @@ static void mp_destroy(struct __mempool_t *mp)
 static khash_t(64) *bsw2_connectivity(const bwtl_t *b)
 {
 	khash_t(64) *h;
-	uint32_t k, l, cntk[4], cntl[4];
+	uint32_t k, l, cntk[4], cntl[4]; // this is fine
 	uint64_t x;
 	khiter_t iter;
 	int j, ret;
@@ -144,17 +156,17 @@ static void cut_tail(bsw2entry_t *u, int T, bsw2entry_t *aux)
 	}
 }
 // remove duplicated cells
-static inline void remove_duplicate(bsw2entry_t *u, khash_t(64) *hash)
+static inline void remove_duplicate(bsw2entry_t *u, khash_t(qintv) *hash)
 {
 	int i, ret, j;
 	khiter_t k;
-	uint64_t key;
-	kh_clear(64, hash);
+	qintv_t key;
+	kh_clear(qintv, hash);
 	for (i = 0; i != u->n; ++i) {
 		bsw2cell_t *p = u->array + i;
 		if (p->ql == 0) continue;
-		key = (uint64_t)p->qk << 32 | p->ql;
-		k = kh_put(64, hash, key, &ret);
+		key.k = p->qk; key.l = p->ql;
+		k = kh_put(qintv, hash, key, &ret);
 		j = -1;
 		if (ret == 0) {
 			if ((uint32_t)kh_value(hash, k) >= p->G) j = i;
@@ -211,7 +223,7 @@ static inline double time_elapse(const struct rusage *curr, const struct rusage
 static void save_hits(const bwtl_t *bwt, int thres, bsw2hit_t *hits, bsw2entry_t *u)
 {
 	int i;
-	uint32_t k;
+	uint32_t k; // this is fine
 	for (i = 0; i < u->n; ++i) {
 		bsw2cell_t *p = u->array + i;
 		if (p->G < thres) continue;
@@ -231,7 +243,8 @@ static void save_hits(const bwtl_t *bwt, int thres, bsw2hit_t *hits, bsw2entry_t
 		}
 	}
 }
-
+/* "narrow hits" are node-to-node hits that have a high score and
+ * are not so repetitive (|SA interval|<=IS). */
 static void save_narrow_hits(const bwtl_t *bwtl, bsw2entry_t *u, bwtsw2_t *b1, int t, int IS)
 {
 	int i;
@@ -255,15 +268,16 @@ static void save_narrow_hits(const bwtl_t *bwtl, bsw2entry_t *u, bwtsw2_t *b1, i
 		}
 	}
 }
-
-int bsw2_resolve_duphits(const bwt_t *bwt, bwtsw2_t *b, int IS)
+/* after this, "narrow SA hits" will be expanded and the coordinates
+ * will be obtained and stored in b->hits[*].k. */
+int bsw2_resolve_duphits(const bntseq_t *bns, const bwt_t *bwt, bwtsw2_t *b, int IS)
 {
-	int i, j, n;
+	int i, j, n, is_rev;
 	if (b->n == 0) return 0;
-	if (bwt) { // convert to chromosomal coordinates if suitable
+	if (bwt && bns) { // convert to chromosomal coordinates if requested
 		int old_n = b->n;
 		bsw2hit_t *old_hits = b->hits;
-		for (i = n = 0; i < b->n; ++i) {
+		for (i = n = 0; i < b->n; ++i) { // compute the memory to allocated
 			bsw2hit_t *p = old_hits + i;
 			if (p->l - p->k + 1 <= IS) n += p->l - p->k + 1;
 			else if (p->G > 0) ++n;
@@ -272,44 +286,52 @@ int bsw2_resolve_duphits(const bwt_t *bwt, bwtsw2_t *b, int IS)
 		b->hits = calloc(b->max, sizeof(bsw2hit_t));
 		for (i = j = 0; i < old_n; ++i) {
 			bsw2hit_t *p = old_hits + i;
-			if (p->l - p->k + 1 <= IS) {
+			if (p->l - p->k + 1 <= IS) { // the hit is no so repetitive
 				bwtint_t k;
+				if (p->G == 0 && p->k == 0 && p->l == 0 && p->len == 0) continue;
 				for (k = p->k; k <= p->l; ++k) {
 					b->hits[j] = *p;
-					b->hits[j].k = bwt_sa(bwt, k);
+					b->hits[j].k = bns_depos(bns, bwt_sa(bwt, k), &is_rev);
 					b->hits[j].l = 0;
+					b->hits[j].is_rev = is_rev;
+					if (is_rev) b->hits[j].k -= p->len - 1;
 					++j;
 				}
 			} else if (p->G > 0) {
 				b->hits[j] = *p;
-				b->hits[j].k = bwt_sa(bwt, p->k);
+				b->hits[j].k = bns_depos(bns, bwt_sa(bwt, p->k), &is_rev);
 				b->hits[j].l = 0;
 				b->hits[j].flag |= 1;
+				b->hits[j].is_rev = is_rev;
+				if (is_rev) b->hits[j].k -= p->len - 1;
 				++j;
 			}
 		}
 		free(old_hits);
 	}
+	for (i = j = 0; i < b->n; ++i) // squeeze out empty elements
+		if (b->hits[i].G) b->hits[j++] = b->hits[i];
+	b->n = j;
 	ks_introsort(hitG, b->n, b->hits);
 	for (i = 1; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
-		if (p->G == 0) break;
 		for (j = 0; j < i; ++j) {
 			bsw2hit_t *q = b->hits + j;
 			int compatible = 1;
-			if (q->G == 0) continue;
+			if (p->is_rev != q->is_rev) continue; // hits from opposite strands are not duplicates
 			if (p->l == 0 && q->l == 0) {
-				int qol = (p->end < q->end? p->end : q->end) - (p->beg > q->beg? p->beg : q->beg);
+				int qol = (p->end < q->end? p->end : q->end) - (p->beg > q->beg? p->beg : q->beg); // length of query overlap
 				if (qol < 0) qol = 0;
 				if ((float)qol / (p->end - p->beg) > MASK_LEVEL || (float)qol / (q->end - q->beg) > MASK_LEVEL) {
 					int64_t tol = (int64_t)(p->k + p->len < q->k + q->len? p->k + p->len : q->k + q->len)
-						- (int64_t)(p->k > q->k? p->k : q->k);
+						- (int64_t)(p->k > q->k? p->k : q->k); // length of target overlap
 					if ((double)tol / p->len > MASK_LEVEL || (double)tol / q->len > MASK_LEVEL)
 						compatible = 0;
 				}
 			}
 			if (!compatible) {
 				p->G = 0;
+				if (q->G2 < p->G2) q->G2 = p->G2;
 				break;
 			}
 		}
@@ -329,18 +351,35 @@ int bsw2_resolve_query_overlaps(bwtsw2_t *b, float mask_level)
 	int i, j, n;
 	if (b->n == 0) return 0;
 	ks_introsort(hitG, b->n, b->hits);
+	{ // choose a random one
+		int G0 = b->hits[0].G;
+		for (i = 1; i < b->n; ++i)
+			if (b->hits[i].G != G0) break;
+		j = (int)(i * drand48());
+		if (j) {
+			bsw2hit_t tmp;
+			tmp = b->hits[0]; b->hits[0] = b->hits[j]; b->hits[j] = tmp;
+		}
+	}
 	for (i = 1; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
 		int all_compatible = 1;
 		if (p->G == 0) break;
 		for (j = 0; j < i; ++j) {
 			bsw2hit_t *q = b->hits + j;
+			int64_t tol = 0;
 			int qol, compatible = 0;
+			float fol;
 			if (q->G == 0) continue;
 			qol = (p->end < q->end? p->end : q->end) - (p->beg > q->beg? p->beg : q->beg);
 			if (qol < 0) qol = 0;
-			if ((float)qol / (p->end - p->beg) < mask_level
-				&& (float)qol / (q->end - q->beg) < mask_level) compatible = 1;
+			if (p->l == 0 && q->l == 0) {
+				tol = (int64_t)(p->k + p->len < q->k + q->len? p->k + p->len : q->k + q->len)
+					- (p->k > q->k? p->k : q->k);
+				if (tol < 0) tol = 0;
+			}
+			fol = (float)qol / (p->end - p->beg < q->end - q->beg? p->end - p->beg : q->end - q->beg);
+			if (fol < mask_level || (tol > 0 && qol < p->end - p->beg && qol < q->end - q->beg)) compatible = 1;
 			if (!compatible) {
 				if (q->G2 < p->G) q->G2 = p->G;
 				all_compatible = 0;
@@ -406,14 +445,15 @@ static void init_bwtsw2(const bwtl_t *target, const bwt_t *query, bsw2stack_t *s
 	u->n++;
 	stack_push0(s, u);
 }
-
-bwtsw2_t **bsw2_core(const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *query, bsw2global_t *pool)
+/* On return, ret[1] keeps not-so-repetitive hits (narrow SA hits); ret[0] keeps all hits (right?) */
+bwtsw2_t **bsw2_core(const bntseq_t *bns, const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *query, bsw2global_t *pool)
 {
 	bsw2stack_t *stack = (bsw2stack_t*)pool->stack;
 	bwtsw2_t *b, *b1, **b_ret;
 	int i, j, score_mat[16], *heap, heap_size, n_tot = 0;
 	struct rusage curr, last;
-	khash_t(64) *rhash, *chash;
+	khash_t(qintv) *rhash;
+	khash_t(64) *chash;
 
 	// initialize connectivity hash (chash)
 	chash = bsw2_connectivity(target);
@@ -422,7 +462,7 @@ bwtsw2_t **bsw2_core(const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *qu
 		for (j = 0; j != 4; ++j)
 			score_mat[i<<2|j] = (i == j)? opt->a : -opt->b;
 	// initialize other variables
-	rhash = kh_init(64);
+	rhash = kh_init(qintv);
 	init_bwtsw2(target, query, stack);
 	heap_size = opt->z;
 	heap = calloc(heap_size, sizeof(int));
@@ -439,7 +479,8 @@ bwtsw2_t **bsw2_core(const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *qu
 	while (!stack_isempty(stack)) {
 		int old_n, tj;
 		bsw2entry_t *v;
-		uint32_t k, l, tcntk[4], tcntl[4];
+		uint32_t tcntk[4], tcntl[4];
+		bwtint_t k, l;
 
 		v = stack_pop(stack); old_n = v->n;
 		n_tot += v->n;
@@ -456,7 +497,7 @@ bwtsw2_t **bsw2_core(const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *qu
 		// get Occ for the DAG
 		bwtl_2occ4(target, v->tk - 1, v->tl, tcntk, tcntl);
 		for (tj = 0; tj != 4; ++tj) { // descend to the children
-			uint32_t qcntk[4], qcntl[4];
+			bwtint_t qcntk[4], qcntl[4];
 			int qj, *curr_score_mat = score_mat + tj * 4;
 			khiter_t iter;
 			bsw2entry_t *u;
@@ -563,12 +604,15 @@ bwtsw2_t **bsw2_core(const bsw2opt_t *opt, const bwtl_t *target, const bwt_t *qu
 		mp_free(stack->pool, v);
 	} // while(top)
 	getrusage(0, &curr);
-	bsw2_resolve_duphits(query, b, opt->is);
-	bsw2_resolve_duphits(query, b1, opt->is);
+	for (i = 0; i < 2; ++i)
+		for (j = 0; j < b_ret[i]->n; ++j)
+			b_ret[i]->hits[j].n_seeds = 0;
+	bsw2_resolve_duphits(bns, query, b, opt->is);
+	bsw2_resolve_duphits(bns, query, b1, opt->is);
 	//fprintf(stderr, "stats: %.3lf sec; %d elems\n", time_elapse(&curr, &last), n_tot);
 	// free
 	free(heap);
-	kh_destroy(64, rhash);
+	kh_destroy(qintv, rhash);
 	kh_destroy(64, chash);
 	stack->pending.n = stack->stack0.n = 0;
 	return b_ret;
